import axios from 'axios'

// import store from '@/store/store'

let ax = axios.create({
  baseURL: 'http://localhost:8000/',
  headers: {'Authorization': 'JWT ' + sessionStorage.getItem('token')}
})

export async function fetchJWT (credentials) {
  const token = (await ax.post('/api-token-auth/', credentials)).data.token
  ax.defaults.headers['Authorization'] = `JWT ${token}`
  return token
}

export async function refreshJWT (token) {
  const newToken = (await ax.post('/api-token-refresh/', {token})).data.token
  ax.defaults.headers['Authorization'] = `JWT ${newToken}`
  return token
}

export async function fetchJWTTimeDelta () {
  return (await ax.get('/api/jwt-time-delta/')).data.timeDelta
}

export async function fetchUserRole () {
  return (await ax.get('/api/user-role/')).data.role
}

export async function fetchStudentSelfData () {
  return (await ax.get('/api/student-page/')).data
}

export async function fetchStudentSubmissions () {
  return (await ax.get('/api/student-submissions/')).data
}

export async function fetchSubscriptions () {
  return (await ax.get('/api/subscription/')).data
}

export async function fetchSubscription (subscriptionPk) {
  return (await ax.get(`/api/subscription/${subscriptionPk}`)).data
}

export async function subscribeTo (type, key, stage) {
  let data = {
    query_type: type
  }

  if (key) {
    data.query_key = key
  }
  if (stage) {
    data.feedback_stage = stage
  }

  return (await ax.post('/api/subscription/', data)).data
}

export async function fetchCurrentAssignment (subscriptionPk) {
  return (await ax.get(`/api/subscription/${subscriptionPk}/assignments/current/`)).data
}

export async function fetchNextAssignment (subscriptionPk) {
  return (await ax.get(`/api/subscription/${subscriptionPk}/assignments/next/`)).data
}

export async function submitFeedbackForAssignment (feedback, assignmentPk) {
  const data = {
    ...feedback,
    assignment_pk: assignmentPk
  }

  return (await ax.post('/api/feedback/', data)).data
}

export async function fetchSubmissionTypes (fields = []) {
  let url = '/api/submissiontype/'
  if (fields.length > 0) {
    url += '?fields=pk,' + fields
  }
  return (await ax.get(url)).data
}

export default ax
