import {types} from './mutations'
import * as api from '@/api'
import router from '@/router/index'

const actions = {
  async getSubscriptions ({ commit }) {
    try {
      const subscriptions = await api.fetchSubscriptions()
      commit(types.SET_SUBSCRIPTIONS, subscriptions)
    } catch (e) {
      console.log(e)
    }
  },
  async subscribeTo ({ commit }, {type, key, stage}) {
    try {
      const subscription = await api.subscribeTo(type, key, stage)
      commit(types.SET_SUBSCRIPTION, subscription)
    } catch (e) {
      console.log(e)
    }
  },
  async updateSubmissionTypes ({ commit }, fields) {
    try {
      const submissionTypes = await api.fetchSubmissionTypes(fields)
      submissionTypes.forEach(type => {
        commit(types.UPDATE_SUBMISSION_TYPE, type)
      })
    } catch (e) {
      console.log(e)
    }
  },
  async getCurrentAssignment ({ commit }, subscriptionPk) {
    try {
      const assignment = await api.fetchCurrentAssignment(subscriptionPk)
      commit(types.UPDATE_ASSIGNMENT, {
        assignment,
        subscriptionPk,
        key: 'currentAssignment'
      })
      return assignment
    } catch (e) {
      console.log(e)
    }
  },
  async getNextAssignment ({ commit }, subscriptionPk) {
    try {
      const assignment = await api.fetchNextAssignment(subscriptionPk)
      commit(types.UPDATE_ASSIGNMENT, {
        assignment,
        subscriptionPk,
        key: 'nextAssignment'
      })
      return assignment
    } catch (e) {
      console.log(e)
    }
  },
  logout ({ commit }, message = '') {
    commit(types.RESET_STATE)
    commit('SET_MESSAGE', message)
    router.push('/')
  }
}

export default actions
