import {fetchJWT, fetchJWTTimeDelta, fetchUserRole, refreshJWT} from '@/api'
import gradySays from '../grady_speak'

function initialState () {
  return {
    token: sessionStorage.getItem('token'),
    tokenCreationTime: 0,
    refreshingToken: false,
    username: '',
    jwtTimeDelta: 0,
    userRole: '',
    message: ''
  }
}

const authentication = {
  state: {
    ...initialState()
  },
  getters: {
    gradySpeak: () => {
      return gradySays[Math.floor(Math.random() * gradySays.length)]
    },
    isStudent: state => {
      return state.userRole === 'Student'
    },
    isTutor: state => {
      return state.userRole === 'Tutor'
    },
    isReviewer: state => {
      return state.userRole === 'Reviewer'
    }
  },
  mutations: {
    'SET_MESSAGE': function (state, message) {
      state.message = message
    },
    'SET_JWT_TOKEN': function (state, token) {
      sessionStorage.setItem('token', token)
      state.token = token
      state.tokenCreationTime = Date.now()
    },
    'SET_JWT_TIME_DELTA': function (state, timeDelta) {
      state.jwtTimeDelta = timeDelta
    },
    'SET_USERNAME': function (state, username) {
      state.username = username
    },
    'SET_USER_ROLE': function (state, userRole) {
      state.userRole = userRole
    },
    'RESET_STATE': function (state) {
      sessionStorage.setItem('token', '')
      Object.assign(state, initialState())
    },
    'SET_REFRESHING_TOKEN': function (state, refreshing) {
      state.refreshingToken = refreshing
    }
  },
  actions: {
    async getJWT (context, credentials) {
      try {
        const token = await fetchJWT(credentials)
        context.commit('SET_USERNAME', credentials.username)
        context.commit('SET_JWT_TOKEN', token)
      } catch (error) {
        console.log(error)
        if (error.response) {
          const errorMsg = 'Unable to log in with provided credentials.'
          context.commit('SET_MESSAGE', errorMsg)
          throw errorMsg
        } else {
          const errorMsg = 'Cannot reach server.'
          context.commit('SET_MESSAGE', errorMsg)
          throw errorMsg
        }
      }
    },
    async refreshJWT ({state, commit, dispatch}) {
      commit('SET_REFRESHING_TOKEN', true)
      try {
        const token = await refreshJWT(state.token)
        commit('SET_JWT_TOKEN', token)
      } catch (err) {
        dispatch('logout')
      } finally {
        commit('SET_REFRESHING_TOKEN', false)
      }
    },
    async getUserRole ({commit}) {
      try {
        const userRole = await fetchUserRole()
        commit('SET_USER_ROLE', userRole)
      } catch (err) {
        commit('SET_MESSAGE', "You've been logged out.")
      }
    },
    async getJWTTimeDelta ({commit}) {
      try {
        const delta = await fetchJWTTimeDelta()
        // multiply by 1000 to convert to ms
        commit('SET_JWT_TIME_DELTA', delta * 1000)
      } catch (err) {
        console.log(err)
      }
    }
  }
}

export default authentication
