import {fetchStudentSelfData, fetchStudentSubmissions} from '../../api'

const studentPage = {
  state: {
    studentName: '',
    exam: {},
    submissionsForList: [],
    submissionData: {},
    visited: {},
    loaded: false
  },
  mutations: {
    'SET_STUDENT_NAME': function (state, name) {
      state.studentName = name
    },
    'SET_EXAM': function (state, exam) {
      state.exam = exam
    },
    'SET_SUBMISSION_TYPES': function (state, submissionTypes) {
      state.submissionTypes = submissionTypes
    },
    'SET_SUBMISSIONS_FOR_LIST': function (state, submissions) {
      state.submissionsForList = submissions
    },
    /**
     * Reduces the array submissionData returned by the /api/student-submissions
     * into an object where the keys are the SubmissionType id's and the values
     * the former array elements. This is done to have direct access to the data
     * via the SubmissionType id.
     */
    'SET_FULL_SUBMISSION_DATA': function (state, submissionData) {
      state.submissionData = submissionData.reduce((acc, cur, index) => {
        acc[cur.type.pk] = cur
        return acc
      }, {})
    },
    'SET_VISITED': function (state, visited) {
      state.visited = { ...state.visited, [visited.index]: visited.visited }
    },
    'SET_LOADED': function (state, loaded) {
      state.loaded = loaded
    }
  },
  actions: {

    async getStudentData (context) {
      try {
        const studentData = await fetchStudentSelfData()
        context.commit('SET_STUDENT_NAME', studentData.name)
        context.commit('SET_EXAM', studentData.exam)
        context.commit('SET_SUBMISSIONS_FOR_LIST', studentData.submissions)
        context.commit('SET_LOADED', true)
      } catch (e) {
        this.$notify({
          title: 'API Fail',
          text: 'Unable to fetch student data',
          type: 'error'
        })
        console.log(e)
      }
    },

    async getStudentSubmissions (context) {
      try {
        const submissions = await fetchStudentSubmissions()
        context.commit('SET_FULL_SUBMISSION_DATA', submissions)
      } catch (e) {
        this.$notify({
          title: 'API Fail',
          text: 'Unable to fetch student submissions',
          type: 'error'
        })
      }
    }
  }
}

export default studentPage
