import Vue from 'vue'
import * as api from '@/api'

function initialState () {
  return {
    assignment: '',
    ui: {
      showEditorOnLine: {},
      selectedCommentOnLine: {}
    },
    orig: {
      rawSubmission: '',
      score: null,
      feedbackLines: {}
    },
    updated: {
      score: null,
      feedbackLines: {}
    }
  }
}

const submissionNotes = {
  namespaced: true,
  state: {
    ...initialState()
  },
  getters: {
    // reduce the string rawSubmission into an object where the keys are the
    // line indexes starting at one and the values the corresponding submission line
    // this makes iterating over the submission much more pleasant
    submission: state => {
      return state.orig.rawSubmission.split('\n').reduce((acc, cur, index) => {
        acc[index + 1] = cur
        return acc
      }, {})
    },
    score: state => {
      return state.updated.score !== null ? state.updated.score : state.orig.score
    }
  },
  mutations: {
    'SET_RAW_SUBMISSION': function (state, submission) {
      state.orig.rawSubmission = submission
    },
    'SET_ORIG_FEEDBACK': function (state, feedback) {
      if (feedback) {
        state.orig.feedbackLines = feedback['feedback_lines'] ? feedback['feedback_lines'] : {}
        state.orig.score = feedback.score
      }
    },
    'UPDATE_FEEDBACK_LINE': function (state, feedback) {
      Vue.set(state.updated.feedbackLines, feedback.lineNo, feedback.comment)
    },
    'UPDATE_FEEDBACK_SCORE': function (state, score) {
      state.updated.score = score
    },
    'DELETE_FEEDBACK_LINE': function (state, lineNo) {
      Vue.delete(state.updated.feedbackLines, lineNo)
    },
    'TOGGLE_EDITOR_ON_LINE': function (state, {lineNo, comment}) {
      Vue.set(state.ui.selectedCommentOnLine, lineNo, comment)
      Vue.set(state.ui.showEditorOnLine, lineNo, !state.ui.showEditorOnLine[lineNo])
    },
    'RESET_STATE': function (state) {
      Object.assign(state, initialState())
    }
  },
  actions: {
    'submitFeedback': async function ({state}, assignment) {
      let feedback = {}
      if (Object.keys(state.updated.feedbackLines).length > 0) {
        feedback['feedback_lines'] = state.updated.feedbackLines
      }
      if (state.orig.score === null && state.updated.score === null) {
        throw new Error('You need to give a score.')
      } else if (state.updated.score !== null) {
        feedback['score'] = state.updated.score
      }
      return api.submitFeedbackForAssignment(feedback, assignment['pk'])
    }
  }
}

export default submissionNotes
