import Vue from 'vue'

import {initialState} from '@/store/store'

export const types = {
  SET_ASSIGNMENT: 'SET_ASSIGNMENT',
  SET_SUBSCRIPTIONS: 'SET_SUBSCRIPTIONS',
  SET_SUBSCRIPTION: 'SET_SUBSCRIPTION',
  UPDATE_SUBMISSION_TYPE: 'UPDATE_SUBMISSION_TYPE',
  UPDATE_ASSIGNMENT: 'UPDATE_ASSIGNMENT',
  UPDATE_NEXT_ASSIGNMENT: 'UPDATE_NEXT_ASSIGNMENT',
  RESET_STATE: 'RESET_STATE',
  SET_LAST_INTERACTION: 'SET_LAST_INTERACTION'
}

const mutations = {
  [types.SET_ASSIGNMENT] (state, assignment) {
    Vue.set(state.assignments, assignment.pk, assignment)
  },
  [types.SET_SUBSCRIPTIONS] (state, subscriptions) {
    state.subscriptions = subscriptions.reduce((acc, curr) => {
      acc[curr['pk']] = curr
      return acc
    }, {})
  },
  [types.SET_SUBSCRIPTION] (state, subscription) {
    Vue.set(state.subscriptions, subscription.pk, subscription)
  },
  [types.UPDATE_SUBMISSION_TYPE] (state, submissionType) {
    const updatedSubmissionType = {
      ...state.submissionTypes[submissionType.pk],
      ...submissionType
    }
    Vue.set(state.submissionTypes, submissionType.pk, updatedSubmissionType)
  },
  [types.UPDATE_ASSIGNMENT] (state, {key, assignment, subscriptionPk}) {
    const submission = assignment.submission
    const feedback = assignment.feedback
    let updatedAssignment = {
      ...state.assignments[assignment.pk],
      ...assignment
    }
    if (feedback) {
      Vue.set(state.feedback, feedback.pk, feedback)
    }
    Vue.set(state.assignments, assignment.pk, updatedAssignment)
    Vue.set(state.submissions, submission.pk, submission)
    Vue.set(state.subscriptions[subscriptionPk], key, updatedAssignment)
  },
  [types.RESET_STATE] (state) {
    Object.assign(state, initialState())
  },
  [types.SET_LAST_INTERACTION] (state) {
    state.lastAppInteraction = Date.now()
  }
}

export default mutations
